#!/usr/bin/env node
"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path = __importStar(require("path"));
var docopt_1 = require("docopt");
var zip_1 = __importDefault(require("@strong-roots-capital/zip"));
var fast_memoize_1 = __importDefault(require("fast-memoize"));
var functional_programming_1 = require("./functional-programming");
var typescript_build_linker_1 = require("./typescript-build-linker");
var utils_1 = require("./utils");
var version_1 = require("./version");
/* eslint-disable @typescript-eslint/no-var-requires */
var compose = require('just-compose');
/* eslint-enable @typescript-eslint/no-var-requires */
var debug = functional_programming_1.traceDebugger(require('debug')('tsl'));
var docstring = function () { return "\nTypeScript Build Linker version " + version_1.version + "\n\nLink TypeScript Project References in a lerna monorepo.\n\nUsage:\n      tsl [--dry-run] <repository>\n"; };
// parseOptions :: CommandLineOptions
var parseOptions = fast_memoize_1.default(function () {
    return docopt_1.docopt(docstring(), {
        help: true,
        version: version_1.version,
        exit: true
    });
});
// FIXME: make the dryRun output more representative of what will be
// written to disk
// isDryRun :: Boolean
var isDryRun = // (): boolean =>
 fast_memoize_1.default(compose(parseOptions, functional_programming_1.prop('--dry-run'), debug('is dry run')));
// repositoryRoot :: string
var repositoryRoot = compose(parseOptions, functional_programming_1.prop('<repository>'));
// absoluteRepositoryRoot :: string
var absoluteRepositoryRoot = compose(repositoryRoot, path.resolve);
// repositoryFile :: string -> string
var repositoryFile = utils_1.join(repositoryRoot());
// lernaJson :: string
var lernaJson = function () {
    return repositoryFile('lerna.json');
};
// lernaPackagesGlob :: File -> Glob[]
var lernaPackagesGlob = fast_memoize_1.default(compose(lernaJson, typescript_build_linker_1.readJson, functional_programming_1.prop('packages'), functional_programming_1.map(repositoryFile), debug('lerna packages glob')));
var zipWithPackageDirectory = compose(lernaPackagesGlob, typescript_build_linker_1.packageJsons, functional_programming_1.map(path.dirname), zip_1.default)();
var lernaPackageDependencies = compose(lernaPackagesGlob, typescript_build_linker_1.internalPackageDependencies, functional_programming_1.map(functional_programming_1.map(typescript_build_linker_1.internalPackagePath(lernaPackagesGlob()))), zipWithPackageDirectory);
// pathInRepository :: File -> File
var pathInRepository = function (file) {
    return path.relative(absoluteRepositoryRoot(), file);
};
// lernaPackages :: string[]
var lernaPackages = fast_memoize_1.default(compose(lernaPackagesGlob, typescript_build_linker_1.packageDirectories, functional_programming_1.map(pathInRepository)));
// linkDependentPackages :: boolean -> Effect
var linkDependentPackages = function (isDryRun) {
    return compose(lernaPackageDependencies, functional_programming_1.map(typescript_build_linker_1.toPackageReferences), debug('lerna package references'), functional_programming_1.map(typescript_build_linker_1.writePackageReferences(isDryRun)), utils_1.JSONstringifyPretty, functional_programming_1.log(isDryRun)(console.log.bind(console)));
};
// linkChildrenPackages :: Effect
var linkChildrenPackages = function (isDryRun) {
    return compose(lernaPackages, typescript_build_linker_1.directoriesContainingPackages, debug('directories containing packages'), typescript_build_linker_1.toParentReferences(lernaPackages()), debug('parent references'), typescript_build_linker_1.writeParentReferences(isDryRun)(repositoryRoot()), utils_1.JSONstringifyPretty, functional_programming_1.log(isDryRun)(console.log.bind(console)));
};
// main :: boolean -> Effect
var main = function () {
    linkDependentPackages(isDryRun())();
    linkChildrenPackages(isDryRun())();
};
main();
//# sourceMappingURL=index.js.map